package com.muller.awarenessapp.activities;

import androidx.annotation.NonNull;
import androidx.appcompat.app.AppCompatActivity;

import android.app.PendingIntent;
import android.content.Intent;
import android.os.Bundle;
import android.util.Log;
import android.view.View;
import android.view.ViewGroup;
import android.widget.AdapterView;
import android.widget.ListView;
import android.widget.Toast;

import com.google.android.gms.awareness.Awareness;
import com.google.android.gms.awareness.fence.AwarenessFence;
import com.google.android.gms.awareness.fence.FenceUpdateRequest;
import com.google.android.gms.awareness.fence.HeadphoneFence;
import com.google.android.gms.awareness.state.HeadphoneState;
import com.google.android.gms.tasks.OnFailureListener;
import com.google.android.gms.tasks.OnSuccessListener;
import com.muller.awarenessapp.Const;
import com.muller.awarenessapp.R;
import com.muller.awarenessapp.adapters.ReleaseAdapter;
import com.muller.awarenessapp.dialogs.ReleaseDialog;
import com.muller.awarenessapp.fences.Release;
import com.muller.awarenessapp.io.Fences;

import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Date;
import java.util.Locale;

/**
 * Activity with list of registered Release fences and button to add a new one
 */
public class ReleaseActivity extends AppCompatActivity implements ReleaseDialog.DialogListener{
    private static final String TAG = "ReleaseActivity";
    private PendingIntent pendingIntent;
    private ArrayList<Release> releases;
    private ReleaseAdapter adapter;
    private Release selected;

    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.activity_release);
        pendingIntent = PendingIntent.getBroadcast(this, Const.PENDING_REQ_CODE, new Intent(Const.FENCE_RECEIVER_ACTION), 0);
        releases = Fences.getInstance().getReleases();
        setAdapter();
    }

    @Override
    protected void onStop() {
        Fences.getInstance().writeFences(Const.RELEASES_FILE, releases);
        super.onStop();
    }

    /**
     * Reaction for add release button, open up dialog
     * @param v View that is triggering this method
     */
    public void registerClick(View v) {
        openDialog();
    }

    /**
     * Method which opens dialog to register new release
     */
    private void openDialog() {
        ReleaseDialog dialog = new ReleaseDialog(null);
        dialog.show(getSupportFragmentManager(), "dialog");
    }

    /**
     * Method which sets listView and its adapter
     */
    private void setAdapter() {
        adapter = new ReleaseAdapter(getApplicationContext(), R.layout.listview_row_music, releases);
        ListView listView = findViewById(R.id.list);
        listView.setAdapter(adapter);
        setListViewListener(listView);
    }

    /**
     * Method which sets listener for listView, creates new dialog for editing if clicked on its
     * item
     * @param listView listView whose listener will be set
     */
    private void setListViewListener(final ListView listView) {
        listView.setOnItemClickListener(new AdapterView.OnItemClickListener() {
            @Override
            public void onItemClick(AdapterView<?> adapterView, View view, int position, long arg) {
                selected = (Release) adapterView.getItemAtPosition(position);
                ReleaseDialog dialog = new ReleaseDialog(selected);
                dialog.show(getSupportFragmentManager(), "dialog");
            }
        });

    }

    /**
     * Method which registers new Release and saves it in file
     * @param release release to be registered and saved
     */
    private void registerFence(final Release release) {
        AwarenessFence headphonesFence = HeadphoneFence.during(HeadphoneState.PLUGGED_IN);
        Awareness.getFenceClient(this).updateFences(new FenceUpdateRequest.Builder()
                .addFence(release.getKey(), headphonesFence, pendingIntent)
                .build())
                .addOnSuccessListener(new OnSuccessListener<Void>() {
                    @Override
                    public void onSuccess(Void aVoid) {
                        adapter.add(release);
                        Fences.getInstance().getFences().add(release);
                        Fences.getInstance().writeFences(Const.RELEASES_FILE, releases);
                        Log.e(TAG, "Fence was successfully registered." + release.getKey());
                    }
                })
                .addOnFailureListener(new OnFailureListener() {
                    @Override
                    public void onFailure(@NonNull Exception e) {
                        Log.e(TAG, "Fence could not be registered: " + e);
                    }
                });
    }

    /**
     * After validation which occurs inside ReleaseDialog class, calculates id for one release
     *  and applies inputs
     * @param album name of the album
     * @param artist name of the artist
     * @param date date of the release
     */
    @Override
    public void applyTexts(String album, String artist, String date) {
        final int lastId = (releases.size() == 0) ? 0 : releases.get(releases.size() - 1).getId() + 1;
        SimpleDateFormat df = new SimpleDateFormat("dd/MM/yyyy", Locale.getDefault());
        Date myDate;
        try {
            myDate = df.parse(date);
            registerFence(new Release(lastId,myDate, artist, album, false));
        } catch (ParseException e) {
            e.printStackTrace();
        }

        ViewGroup vg = findViewById (R.id.release_layout);
        vg.invalidate();
    }

    /**
     * After validation which occurs inside ReleaseDialog class, edits existing record of
     * release and applies inputs
     * @param album name of the album
     * @param artist name of the artist
     * @param date date of the release
     */
    @Override
    public void editTexts(String album, String artist, String date) {
        SimpleDateFormat df = new SimpleDateFormat("dd/MM/yyyy", Locale.getDefault());
        Date myDate;
        try {
            myDate = df.parse(date);
            selected.setArtist(artist);
            selected.setAlbum(album);
            selected.setDate(myDate);
            adapter.notifyDataSetChanged();
        } catch (ParseException e) {
            Toast.makeText(this, "Cannot parse date", Toast.LENGTH_SHORT).show();
            e.printStackTrace();
        }
    }
}