package com.muller.awarenessapp.activities;

import androidx.annotation.NonNull;
import androidx.appcompat.app.AppCompatActivity;
import androidx.core.app.ActivityCompat;
import androidx.core.content.ContextCompat;

import android.Manifest;
import android.app.Activity;
import android.app.AlertDialog;
import android.app.PendingIntent;
import android.content.DialogInterface;
import android.content.Intent;
import android.content.pm.PackageManager;
import android.os.Build;
import android.os.Bundle;
import android.util.Log;
import android.view.View;
import android.widget.AdapterView;
import android.widget.ListView;
import android.widget.Toast;

import com.google.android.gms.awareness.Awareness;
import com.google.android.gms.awareness.fence.AwarenessFence;
import com.google.android.gms.awareness.fence.FenceUpdateRequest;
import com.google.android.gms.awareness.fence.LocationFence;
import com.google.android.gms.tasks.OnFailureListener;
import com.google.android.gms.tasks.OnSuccessListener;
import com.muller.awarenessapp.Const;
import com.muller.awarenessapp.R;
import com.muller.awarenessapp.adapters.ShoppingLocationAdapter;
import com.muller.awarenessapp.dialogs.LocationDialog;
import com.muller.awarenessapp.fences.ShoppingLocation;
import com.muller.awarenessapp.io.Fences;
import java.util.ArrayList;
import java.util.Comparator;

/**
 * Activity with list of registered Shopping fences location and button to add a new one
 */
public class ShoppingActivity extends AppCompatActivity implements LocationDialog.DialogListener {
    private static final String TAG = "ShoppingActivity";
    private static final int requestCode = 101;
    private PendingIntent pendingIntent;
    private ArrayList<ShoppingLocation> shoppingLocations;
    private ShoppingLocationAdapter adapter;
    private ShoppingLocation selected;

    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.activity_shopping);
        Intent intent = new Intent(Const.FENCE_RECEIVER_ACTION);
        pendingIntent = PendingIntent.getBroadcast(this, Const.PENDING_REQ_CODE, intent, 0);
        shoppingLocations = Fences.getInstance().getShoppingLocations();
        setAdapter();
    }

    /**
     * Reaction for add location button, open up dialog
     * @param v View that is triggering this method
     */
    public void addPlaceClick(View v) {
        openDialog();
    }

    /**
     * Method which opens dialog to register new release
     */
    private void openDialog() {
        LocationDialog dialog = new LocationDialog(null);
        dialog.show(getSupportFragmentManager(), "dialog");
    }

    /**
     * Method which sets listView and its adapter and sorts adapter
     */
    private void setAdapter() {
        adapter = new ShoppingLocationAdapter(getApplicationContext(), R.layout.listview_row_location, shoppingLocations);
        adapter.sort(new Comparator<ShoppingLocation>() {
            @Override
            public int compare(ShoppingLocation l1, ShoppingLocation l2) {
                return (l1.getName()).compareTo(l2.getName());
            }
        });
        ListView listView = findViewById(R.id.lvPlaces);
        listView.setAdapter(adapter);
        setListViewListener(listView);
    }

    /**
     * Method which sets listener for listView, opens new Activity if clicked on its
     * item, adds data to Intent based on the item clicked
     * @param listView listView whose listener will be set
     */
    private void setListViewListener(final ListView listView) {
        listView.setOnItemClickListener(new AdapterView.OnItemClickListener() {
            @Override
            public void onItemClick(AdapterView<?> adapterView, View view, int position, long arg) {
                selected = (ShoppingLocation) adapterView.getItemAtPosition(position);
                Intent locationInfo = new Intent(ShoppingActivity.this, ShoppingInfoActivity.class);
                locationInfo.putExtra("Location", selected);
                startActivityForResult(locationInfo,requestCode);
            }
        });

    }

    /**
     * Method which registers new Shopping location fence and saves it in file, checks required permissions
     * @param shoppingLocation Shopping location to be registered and saved
     */
    private void registerFence(final ShoppingLocation shoppingLocation) {
        if (ContextCompat.checkSelfPermission(this, Manifest.permission.ACCESS_FINE_LOCATION) !=
                PackageManager.PERMISSION_GRANTED) {
            if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.R) {
                AlertDialog alertDialog = new AlertDialog.Builder(this, R.style.AlertDialogTheme).create();
                alertDialog.setTitle("Permission required");
                alertDialog.setMessage("Please navigate to apps settings and make sure to choose Allow all the time on Location permission");
                alertDialog.setButton(AlertDialog.BUTTON_NEUTRAL,  "OK",
                        new DialogInterface.OnClickListener() {
                            public void onClick(DialogInterface dialog, int which) {
                                dialog.dismiss();
                            }
                        });
                alertDialog.show();
            }
            else {
                ActivityCompat.requestPermissions(
                        this,
                        new String[]{Manifest.permission.ACCESS_FINE_LOCATION},
                        Const.LOCATION_REQ_CODE
                );
            }
        }
        else {
            AwarenessFence locationFence = LocationFence.in(shoppingLocation.getLongitude(), shoppingLocation.getLatitude(), Fences.getInstance().getRadius(), 1);

            Awareness.getFenceClient(this).updateFences(new FenceUpdateRequest.Builder()
                    .addFence(shoppingLocation.getKey(), locationFence, pendingIntent)
                    .build())
                    .addOnSuccessListener(new OnSuccessListener<Void>() {
                        @Override
                        public void onSuccess(Void aVoid) {
                            Fences.getInstance().getFences().remove(shoppingLocation);
                            Fences.getInstance().getFences().add(shoppingLocation);
                            Fences.getInstance().writeFences(Const.SHOP_LOC_FILE, shoppingLocations);
                            Log.e(TAG, "Fence was successfully registered." + shoppingLocation.getKey() + " name: " + shoppingLocation.getName() +  ", long: " + shoppingLocation.getLongitude() + ", lati: " + shoppingLocation.getLatitude());
                        }
                    })
                    .addOnFailureListener(new OnFailureListener() {
                        @Override
                        public void onFailure(@NonNull Exception e) {
                            Log.e(TAG, "Fence could not be registered: " + e);
                        }
                    });
        }
    }

    /**
     * Methods which edits existing location
     * @param editedLocation location to be edited
     */
    private void editLocation(ShoppingLocation editedLocation) {
        selected.setId(editedLocation.getId());
        selected.setName(editedLocation.getName());
        selected.setLongitude(editedLocation.getLongitude());
        selected.setLatitude(editedLocation.getLatitude());
        selected.setProducts(editedLocation.getProducts());
        adapter.notifyDataSetChanged();
        if (selected.getProducts().size() > 0) {
            registerFence(selected);
        }
    }


    /**
     * Method gets data from ShoppingInfoActivity class
     * @param requestCode The integer request code originally supplied to startActivityForResult(), allowing you to identify who this result came from
     * @param resultCode The integer result code returned by the child activity through its setResult()
     * @param data An Intent, which can return result data to the caller (various data can be attached to Intent "extras")
     */
    @Override
    public void onActivityResult(int requestCode, int resultCode, Intent data) {
        super.onActivityResult(requestCode, resultCode, data);
        if (requestCode == ShoppingActivity.requestCode) {
            if (resultCode == Activity.RESULT_OK) {
                ShoppingLocation editedLocation = (ShoppingLocation) data.getSerializableExtra("Location");
                if (editedLocation != null) {
                    editLocation(editedLocation);
                }
            }

        }
    }

    /**
     * After validation which occurs inside LocationDialog class, calculates id for one location
     *  and applies inputs
     * @param name name of the location
     * @param longitude longitude
     * @param latitude latitude
     */
    @Override
    public void applyTexts(String name, double longitude, double latitude) {
        int lastId = (shoppingLocations.size() == 0) ? 0 : shoppingLocations.get(shoppingLocations.size() - 1).getId() + 1;
        ShoppingLocation shoppingLocation = new ShoppingLocation(lastId, name, longitude, latitude);
        adapter.add(shoppingLocation);
        Fences.getInstance().writeFences(Const.SHOP_LOC_FILE, shoppingLocations);
    }
}