package com.muller.awarenessapp.activities;

import androidx.annotation.NonNull;
import androidx.appcompat.app.AppCompatActivity;
import androidx.core.app.ActivityCompat;
import androidx.core.content.ContextCompat;

import android.Manifest;
import android.app.Activity;
import android.app.PendingIntent;
import android.content.Intent;
import android.content.pm.PackageManager;
import android.os.Bundle;
import android.util.Log;
import android.view.View;
import android.view.ViewGroup;
import android.widget.AdapterView;
import android.widget.ListView;

import com.google.android.gms.awareness.Awareness;
import com.google.android.gms.awareness.fence.AwarenessFence;
import com.google.android.gms.awareness.fence.FenceUpdateRequest;
import com.google.android.gms.awareness.fence.LocationFence;
import com.google.android.gms.tasks.OnFailureListener;
import com.google.android.gms.tasks.OnSuccessListener;
import com.muller.awarenessapp.Const;
import com.muller.awarenessapp.R;
import com.muller.awarenessapp.adapters.TodoLocationAdapter;
import com.muller.awarenessapp.dialogs.LocationDialog;
import com.muller.awarenessapp.fences.TodoLocation;
import com.muller.awarenessapp.io.Fences;

import java.util.ArrayList;

/**
 * Activity with list of registered To do fences location and button to add a new one
 */
public class TodoActivity extends AppCompatActivity implements LocationDialog.DialogListener{
    private static final String TAG = "TodoActivity";
    private static final int requestCode = 101;
    private ArrayList<TodoLocation> locations;
    private TodoLocationAdapter adapter;
    private TodoLocation selected;
    private PendingIntent pendingIntent;

    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.activity_todo);
        Intent intent = new Intent(Const.FENCE_RECEIVER_ACTION);
        pendingIntent = PendingIntent.getBroadcast(this, 1, intent, 0);
        locations = Fences.getInstance().getTodoLocations();
        setAdapter();
    }

    /**
     * Reaction for add location button, open up dialog
     * @param v View that is triggering this method
     */
    public void addLocationClick(View v) {
        openDialog();
    }

    /**
     * Method which opens dialog to register new release
     */
    private void openDialog() {
        LocationDialog dialog = new LocationDialog(null);
        dialog.show(getSupportFragmentManager(), "dialog");
    }

    /**
     * Method which sets listView and its adapter
     */
    private void setAdapter() {
        adapter = new TodoLocationAdapter(getApplicationContext(), R.layout.listview_row_location, locations);
        ListView listView = findViewById(R.id.lvLocations);
        listView.setAdapter(adapter);
        setListViewListener(listView);
    }

    /**
     * Method which sets listener for listView, opens new Activity if clicked on its
     * item, adds data to Intent based on the item clicked
     * @param listView listView whose listener will be set
     */
    private void setListViewListener(final ListView listView) {
        listView.setOnItemClickListener(new AdapterView.OnItemClickListener() {
            @Override
            public void onItemClick(AdapterView<?> adapterView, View view, int position, long arg) {
                selected = (TodoLocation) adapterView.getItemAtPosition(position);
                Intent locationInfo = new Intent(TodoActivity.this, TodoInfoActivity.class);
                locationInfo.putExtra("Location", selected);
                startActivityForResult(locationInfo,requestCode);
            }
        });
    }


    /**
     * Method which registers new To do location fence and saves it in file, checks required permissions
     * @param location To do location to be registered and saved
     */
    private void registerFence(final TodoLocation location) {
        if (ContextCompat.checkSelfPermission(this, Manifest.permission.ACCESS_FINE_LOCATION) !=
                PackageManager.PERMISSION_GRANTED) {
            ActivityCompat.requestPermissions(
                    this,
                    new String[]{Manifest.permission.ACCESS_FINE_LOCATION},
                    Const.LOCATION_REQ_CODE
            );
        }
        else {
            AwarenessFence locationFence = LocationFence.in(location.getLongitude(), location.getLatitude(), Fences.getInstance().getRadius(), 1);

            Awareness.getFenceClient(this).updateFences(new FenceUpdateRequest.Builder()
                    .addFence(location.getKey(), locationFence, pendingIntent)
                    .build())
                    .addOnSuccessListener(new OnSuccessListener<Void>() {
                        @Override
                        public void onSuccess(Void aVoid) {
                            //adapter.add(location);
                            Fences.getInstance().getFences().add(location);
                            Fences.getInstance().writeFences(Const.TODO_LOC_FILE, locations);
                            Log.e(TAG, "Fence was successfully registered." + "id: " + location.getKey() +  " name: " + location.getName() +  ", long: " + location.getLongitude() + ", lati: " + location.getLatitude());
                        }
                    })
                    .addOnFailureListener(new OnFailureListener() {
                        @Override
                        public void onFailure(@NonNull Exception e) {
                            Log.e(TAG, "Fence could not be registered: " + e);
                        }
                    });
        }
    }

    /**
     * Methods which edits existing location
     * @param editedLocation location to be edited
     */
    private void editLocation(TodoLocation editedLocation) {
        selected.setId(editedLocation.getId());
        selected.setName(editedLocation.getName());
        selected.setLongitude(editedLocation.getLongitude());
        selected.setLatitude(editedLocation.getLatitude());
        selected.setTodoTasks(editedLocation.getTodoTasks());
        adapter.notifyDataSetChanged();

        if (selected.getTodoTasks().size() > 0) {
            registerFence(selected);
        }
    }

    /**
     * Method gets data from TodoInfoActivity class
     * @param requestCode The integer request code originally supplied to startActivityForResult(), allowing you to identify who this result came from
     * @param resultCode The integer result code returned by the child activity through its setResult()
     * @param data An Intent, which can return result data to the caller (various data can be attached to Intent "extras")
     */
    @Override
    public void onActivityResult(int requestCode, int resultCode, Intent data) {
        super.onActivityResult(requestCode, resultCode, data);
        if (requestCode == TodoActivity.requestCode) {
            if (resultCode == Activity.RESULT_OK) {
                TodoLocation editedLocation = (TodoLocation) data.getSerializableExtra("Location");
                if (editedLocation != null) {
                    editLocation(editedLocation);
                }
            }

        }
    }


    /**
     * After validation which occurs inside LocationDialog class, calculates id for one location
     *  and applies inputs
     * @param name name of the location
     * @param longitude longitude
     * @param latitude latitude
     */
    @Override
    public void applyTexts(String name, double longitude, double latitude) {
        int lastId = (locations.size() == 0) ? 0 : locations.get(locations.size() - 1).getId() + 1;
        Log.e(TAG, "applyTexts: LAST ID " + lastId);
        TodoLocation location = new TodoLocation(lastId, name, longitude, latitude);
        adapter.add(location);
        Fences.getInstance().writeFences(Const.TODO_LOC_FILE, locations);

        ViewGroup vg = findViewById(R.id.todo_layout);
        vg.invalidate();
    }
}