package com.muller.awarenessapp.dialogs;

import android.app.Activity;
import android.app.AlertDialog;
import android.app.Dialog;
import android.content.Context;
import android.content.DialogInterface;
import android.content.Intent;
import android.os.Bundle;
import android.view.LayoutInflater;
import android.view.View;
import android.widget.Button;
import android.widget.EditText;
import android.widget.ImageView;
import android.widget.Toast;

import androidx.annotation.NonNull;
import androidx.annotation.Nullable;
import androidx.appcompat.app.AppCompatDialogFragment;

import com.muller.awarenessapp.activities.MapsActivity;
import com.muller.awarenessapp.R;
import com.muller.awarenessapp.fences.Location;

/**
 * Class represents dialog with inputs for location
 */
public class LocationDialog extends AppCompatDialogFragment {
    private static final int requestCode = 102;
    private EditText etName;
    private EditText etLongitudeLatitude;
    private Button btnMaps;
    private ImageView ivInfo;
    private DialogListener listener;
    private Context context;
    private Location location;

    public LocationDialog(Location location) {
        this.location = location;
    }

    /**
     * Based on the nullability of Location object, either creates dialog for new location or
     * one for editing existing location
     * @param savedInstanceState if the fragment is being re-created from a previous saved state, this is the state.
     * @return created dialog
     */
    @NonNull
    @Override
    public android.app.Dialog onCreateDialog(@Nullable Bundle savedInstanceState) {
        LayoutInflater inflater = getActivity().getLayoutInflater();
        View view = inflater.inflate(R.layout.dialog_location, null);

        etName = view.findViewById(R.id.etName);
        etLongitudeLatitude = view.findViewById(R.id.etLongLat);
        ivInfo = view.findViewById(R.id.ivInfo);
        btnMaps = view.findViewById(R.id.btnMaps);

        AlertDialog.Builder builder = new AlertDialog.Builder(getActivity(), R.style.AlertDialogTheme);
        builder.setView(view);

        btnMaps.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View view) {
                Intent intent = new Intent(context, MapsActivity.class);
                startActivityForResult(intent,requestCode);

            }
        });

        ivInfo.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View view) {
                AlertDialog alertDialog = new AlertDialog.Builder(context, R.style.AlertDialogTheme).create();
                alertDialog.setTitle("Location information");
                alertDialog.setMessage("Please enter first longitude and then latitude separated with comma.\nFor example: 43.737167, 7.424804");
                alertDialog.setButton(AlertDialog.BUTTON_NEUTRAL, "OK",
                        new DialogInterface.OnClickListener() {
                            public void onClick(DialogInterface dialog, int which) {
                                dialog.dismiss();
                            }
                        });
                alertDialog.show();
            }
        });

        if (location == null) {
            builder.setTitle("Add new place")
                    .setPositiveButton("ADD", new DialogInterface.OnClickListener() {
                        @Override
                        public void onClick(DialogInterface dialogInterface, int i) {
                        }
                    })
                    .setNegativeButton("CANCEL", null);
        }
        else {
            etName.setText(location.getName());
            etLongitudeLatitude.setText(location.getLongLatString());

            builder.setTitle("Edit place")
                    .setPositiveButton("SAVE", new DialogInterface.OnClickListener() {
                        @Override
                        public void onClick(DialogInterface dialogInterface, int i) {

                        }
                    })
                    .setNegativeButton("CANCEL", null);
        }
        return builder.create();
    }

    /**
     * Based on the input, method keeps dialog shown and warn about invalid input or
     * calls one of the interface methods to pass data to Activity and dismiss dialog
     */
    @Override
    public void onResume() {
        super.onResume();

        final AlertDialog d = (AlertDialog)getDialog();
        if (d != null) {
            Button positiveButton = (Button) d.getButton(Dialog.BUTTON_POSITIVE);
            positiveButton.setOnClickListener(new View.OnClickListener() {
                @Override
                public void onClick(View view) {
                    String name = etName.getText().toString();
                    String longitudeLatitude = etLongitudeLatitude.getText().toString();
                    String[] split = longitudeLatitude.split(",");

                    if (split.length != 2 || name.isEmpty() || !(name.trim().length() > 0)) {
                        Toast.makeText(context, "Invalid input", Toast.LENGTH_SHORT).show();
                        return;
                    }

                    try {
                        double longitude = Double.parseDouble(split[0]);
                        double latitude = Double.parseDouble(split[1]);
                        listener.applyTexts(name, longitude, latitude);
                    } catch (NumberFormatException nfe) {
                        Toast.makeText(context, "Invalid input", Toast.LENGTH_SHORT).show();
                        return;
                    }

                    d.dismiss();
                }
            });
        }
    }

    @Override
    public void onAttach(@NonNull Context context) {
        super.onAttach(context);
        this.context = context;
        try {
            listener = (DialogListener)context;
        } catch (Exception e) {
            e.printStackTrace();
        }
    }

    /**
     * Method gets data from MapsActivity class
     * @param requestCode The integer request code originally supplied to startActivityForResult(), allowing you to identify who this result came from
     * @param resultCode The integer result code returned by the child activity through its setResult()
     * @param data An Intent, which can return result data to the caller (various data can be attached to Intent "extras")
     */
    @Override
    public void onActivityResult(int requestCode, int resultCode, @Nullable Intent data) {
        super.onActivityResult(requestCode, resultCode, data);
        if (requestCode == LocationDialog.requestCode) {
            if (resultCode == Activity.RESULT_OK) {
                String s = data.getStringExtra("latLng");
                etLongitudeLatitude.setText(s);

            }
        }
    }

    /**
     * Interface for communication with Activity
     */
    public interface DialogListener {
        void applyTexts(String name, double longitude, double latitude);
    }
}
