package com.muller.awarenessapp.dialogs;

import androidx.annotation.NonNull;
import androidx.annotation.Nullable;
import androidx.appcompat.app.AlertDialog;
import androidx.appcompat.app.AppCompatDialogFragment;

import android.app.Dialog;
import android.content.Context;
import android.content.DialogInterface;
import android.os.Bundle;
import android.util.Log;
import android.view.LayoutInflater;
import android.view.View;
import android.widget.Button;
import android.widget.EditText;
import android.widget.Toast;

import com.muller.awarenessapp.R;
import com.muller.awarenessapp.fences.Release;

public class ReleaseDialog extends AppCompatDialogFragment {
    private static final String TAG = "ReleaseDialog";
    private EditText etAlbum;
    private EditText etArtist;
    private EditText etDate;
    private ReleaseDialog.DialogListener listener;
    private Release release;
    private Context context;

    public ReleaseDialog(Release release) {
        this.release = release;
    }

    /**
     * Based on the nullability of Release object, either creates dialog for new release or
     * one for editing existing release
     * @param savedInstanceState if the fragment is being re-created from a previous saved state, this is the state.
     * @return created dialog
     */
    @NonNull
    @Override
    public android.app.Dialog onCreateDialog(@Nullable Bundle savedInstanceState) {
        LayoutInflater inflater = getActivity().getLayoutInflater();
        View view = inflater.inflate(R.layout.dialog_release, null);

        etAlbum = view.findViewById(R.id.etAlbum);
        etArtist = view.findViewById(R.id.etArtist);
        etDate = view.findViewById(R.id.etDate);

        AlertDialog.Builder builder = new AlertDialog.Builder(getActivity(), R.style.AlertDialogTheme);
        builder.setView(view);

        if (release == null) {
            builder.setTitle("Add new release")
                    .setPositiveButton("ADD", new DialogInterface.OnClickListener() {
                        @Override
                        public void onClick(DialogInterface dialogInterface, int i) {
                        }
                    })
                    .setNegativeButton("CANCEL", null);
        }
        else {
            etAlbum.setText(release.getAlbum());
            etArtist.setText(release.getArtist());
            etDate.setText(release.getStringDate());

            builder.setTitle("Edit release")
                    .setPositiveButton("SAVE", new DialogInterface.OnClickListener() {
                        @Override
                        public void onClick(DialogInterface dialogInterface, int i) {
                        }
                    })
                    .setNegativeButton("CANCEL", null);
        }

        return builder.create();
    }

    /**
     * Based on the input, method keeps dialog shown and warn about invalid input or
     * calls one of the interface methods to pass data to Activity and dismiss dialog
     */
    @Override
    public void onResume() {
        super.onResume();

        final AlertDialog d = (AlertDialog)getDialog();
        if(d != null) {
            Button positiveButton = (Button) d.getButton(Dialog.BUTTON_POSITIVE);
            positiveButton.setOnClickListener(new View.OnClickListener()
            {
                @Override
                public void onClick(View v) {
                    String album = etAlbum.getText().toString();
                    String artist = etArtist.getText().toString();
                    String date = etDate.getText().toString();


                    if (!isInputValid(album, artist, date)) {
                        Toast.makeText(context, "Input is invalid", Toast.LENGTH_SHORT).show();
                    }
                    else {
                        if (release == null) {
                            listener.applyTexts(album, artist, date);
                        }
                        else {
                            listener.editTexts(album, artist, date);
                        }
                        d.dismiss();
                    }
                }
            });
        }
    }

    @Override
    public void onAttach(@NonNull Context context) {
        super.onAttach(context);
        try {
            this.context = context;
            this.listener = (ReleaseDialog.DialogListener) context;
        } catch (Exception e) {
            e.printStackTrace();
        }
    }

    /**
     * Checks the validation of inputs for release
     * @param album name of the album
     * @param artist name of the artist
     * @param date date of the release
     * @return true if valid, otherwise false
     */
    private boolean isInputValid(String album, String artist, String date) {
        Log.e(TAG, "isInputValid: " + artist + " " + album + " " + date);
        if (artist.isEmpty() || album.isEmpty() || date.isEmpty()) {
            return false;
        }

        return date.matches("^(0?[1-9]|[12][0-9]|3[01])[\\/\\-](0?[1-9]|1[012])[\\/\\-]\\d{4}$");
    }

    /**
     * Interface for communication with Activity
     */
    public interface DialogListener {
        void applyTexts(String album, String artist, String date);
        void editTexts(String album, String artist, String date);
    }
}