package com.muller.awarenessapp.io;

import android.app.PendingIntent;
import android.content.Context;
import android.content.Intent;
import android.util.Log;

import com.muller.awarenessapp.Const;
import com.muller.awarenessapp.fences.IFence;
import com.muller.awarenessapp.fences.ShoppingLocation;
import com.muller.awarenessapp.fences.Release;
import com.muller.awarenessapp.fences.TodoLocation;

import java.io.BufferedReader;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStreamReader;
import java.io.ObjectInputStream;
import java.io.ObjectOutputStream;
import java.util.ArrayList;


/**
 * Class which servers as a bridge to the files where fences are stored
 * Basically like an interface for a database
 */
public class Fences {
    private static final String TAG = "Fences";
    private static Fences instance;
    private ArrayList<Release> releases;
    private ArrayList<ShoppingLocation> shoppingLocations;
    private ArrayList<TodoLocation> todoLocations;
    private Context context;
    private PendingIntent pendingIntent;
    private boolean isDebug;
    private int locationRadius;

    private ArrayList<IFence> fences;

    private Fences() {
        releases = new ArrayList<Release>();
        shoppingLocations = new ArrayList<ShoppingLocation>();
        todoLocations = new ArrayList<TodoLocation>();
        fences = new ArrayList<>();
        isDebug = false;
    }

    public static Fences getInstance() {
        if (instance == null) {
            instance = new Fences();
        }
        return instance;
    }


    public ArrayList<Release> getReleases() {
        return releases;
    }

    public ArrayList<ShoppingLocation> getShoppingLocations() {
        return shoppingLocations;
    }

    public ArrayList<TodoLocation> getTodoLocations() {
        return todoLocations;
    }

    public ArrayList<IFence> getFences() {
        return fences;
    }

    public boolean isDebug() {
        return isDebug;
    }

    public void setDebug(boolean debug) {
        isDebug = debug;
    }

    public void setRadius(int radius) {
        this.locationRadius = radius;
    }

    public int getRadius() {
        if (locationRadius == 0) {
            return Const.DEF_RADIUS;
        }
        return this.locationRadius;
    }

    /**
     * Read all fences and settings from their specific files
     * @param context context where method is called
     */
    public void readFences(Context context) {
        this.context = context;

        releases = readFence(Const.RELEASES_FILE, releases);
        shoppingLocations = readFence(Const.SHOP_LOC_FILE, shoppingLocations);
        todoLocations = readFence(Const.TODO_LOC_FILE, todoLocations);

        readSettings();
    }

    /**
     * Registers all fences
     * @param context context where method is called
     */
    public void registerFences(Context context) {
        pendingIntent = PendingIntent.getBroadcast(context, Const.PENDING_REQ_CODE, new Intent(Const.FENCE_RECEIVER_ACTION), 0);

        for (IFence fence : fences) {
            fence.registerFence(context, pendingIntent);
        }
    }


    /**
     * Reads from settings file
     */
    private void readSettings() {
        FileInputStream fis = null;
        try {
            fis = context.openFileInput(Const.SETTINGS_FILE);
            InputStreamReader isr = new InputStreamReader(fis);
            BufferedReader reader = new BufferedReader(isr);

            String line;
            while ((line = reader.readLine()) != null) {
                Log.e(TAG, "readSettings: " + line);
                String[] split = line.split(":");

                this.setRadius(Integer.parseInt(split[1]));
            }
        } catch (IOException e) {
            e.printStackTrace();
        } finally {
            if (fis != null) {
                try {
                    fis.close();
                } catch (IOException e) {
                    e.printStackTrace();
                }
            }
        }
    }

    /**
     * Writes settings to settings file
     * @param locationRadius location radius to be written
     */
    public void writeSettings(int locationRadius) {
        FileOutputStream fos = null;
        String s = "rad:" + locationRadius;
        try {
            fos = context.openFileOutput(Const.SETTINGS_FILE, Context.MODE_PRIVATE);
            fos.write(s.getBytes());
        } catch (FileNotFoundException e) {
            Log.e(TAG, "writeSettings: " + e.getMessage());
        } catch (IOException e) {
            Log.e(TAG, "writeSettings: " + e.getMessage());
        } finally {
            if (fos != null) {
                try {
                    fos.close();
                } catch (IOException e) {
                    e.printStackTrace();
                }
            }
        }
    }

    /**
     * Read specific type of fence
     * @param fileName file from fences will be read
     * @param listToFill list where fences objects will be stored
     * @param <T> type of fence
     * @return list of fences
     */
    private <T extends IFence> ArrayList<T> readFence(String fileName, ArrayList<T> listToFill) {
        FileInputStream fis = null;
        try {
            fis = context.openFileInput(fileName);
            ObjectInputStream ois = new ObjectInputStream(fis);
            Object list = ois.readObject();
            if (list instanceof ArrayList) {
                listToFill = (ArrayList<T>) list;
                fences.addAll(listToFill);
            }
        } catch (IOException e) {
            e.printStackTrace();
        } catch (ClassNotFoundException e) {
            e.printStackTrace();
        } finally {
            if (fis != null) {
                try {
                    fis.close();
                } catch (IOException e) {
                    e.printStackTrace();
                }
            }
        }
        return listToFill;
    }

    /**
     * Writes fences to specific file
     * @param fileName file where fences will be written
     * @param listToWrite list of fences to be written
     * @param <T> type of fences
     */
    public <T> void writeFences(String fileName, ArrayList<T> listToWrite) {
        FileOutputStream fos = null;
        try {
            fos = context.openFileOutput(fileName, Context.MODE_PRIVATE);
            ObjectOutputStream os = new ObjectOutputStream(fos);
            os.writeObject(listToWrite);
            os.close();
        } catch (FileNotFoundException e) {
            e.printStackTrace();
        } catch (IOException e) {
            e.printStackTrace();
        } finally {
            if (fos != null) {
                try {
                    fos.close();
                } catch (IOException e) {
                    e.printStackTrace();
                }
            }
        }
    }
}